import tensorflow as tf
from tensorflow.keras import layers
from tensorflow import keras

class OffensiveCNN2DModel:
    def __init__(self, args, embedding_matrix=None):
        filter_sizes = [1, 2, 3, 5]
        num_filters = 32

        inp = tf.keras.Input(shape=(None,), dtype="int64", name="input")
        x = layers.Embedding(args.max_features, args.embed_size,
                             embeddings_initializer=keras.initializers.Constant(embedding_matrix), trainable=False,
                             name="embedding_layer")(inp)
        x = layers.SpatialDropout1D(0.4, name="spatial_dropout_layer")(x)
        x = layers.Reshape((args.max_len, args.embed_size, 1), name="reshape_layer")(x)

        conv_0 = layers.Conv2D(num_filters, kernel_size=(filter_sizes[0], args.embed_size), kernel_initializer='normal',
                        activation='elu', name="conv0_layer")(x)
        conv_1 = layers.Conv2D(num_filters, kernel_size=(filter_sizes[1], args.embed_size), kernel_initializer='normal',
                        activation='elu', name="conv1_layer")(x)
        conv_2 = layers.Conv2D(num_filters, kernel_size=(filter_sizes[2], args.embed_size), kernel_initializer='normal',
                        activation='elu', name="conv2_layer")(x)
        conv_3 = layers.Conv2D(num_filters, kernel_size=(filter_sizes[3], args.embed_size), kernel_initializer='normal',
                        activation='elu', name="conv3_layer")(x)

        maxpool_0 = layers.MaxPool2D(pool_size=(args.max_len - filter_sizes[0] + 1, 1), name="pool0_layer")(conv_0)
        maxpool_1 = layers.MaxPool2D(pool_size=(args.max_len - filter_sizes[1] + 1, 1), name="pool1_layer")(conv_1)
        maxpool_2 = layers.MaxPool2D(pool_size=(args.max_len - filter_sizes[2] + 1, 1), name="pool2_layer")(conv_2)
        maxpool_3 = layers.MaxPool2D(pool_size=(args.max_len - filter_sizes[3] + 1, 1), name="pool3_layer")(conv_3)

        z = layers.Concatenate(axis=1, name="conc_layer")([maxpool_0, maxpool_1, maxpool_2, maxpool_3])
        z = layers.Flatten(name="flatten_layer")(z)
        z = layers.Dropout(0.1, name="dropout_layer")(z)
        outp = layers.Dense(args.num_classes, activation="softmax", name="dense_predictions")(z)
        self.model = tf.keras.Model(inputs=inp, outputs=outp, name="cnn_model")



import pandas as pd
from sklearn.model_selection import train_test_split

from experiments.OLID.olid_config import args
from offensive_nn.offensive_nn_model import OffensiveNNModel
from offensive_nn.util.label_converter import encode, decode
from offensive_nn.util.print_stat import print_information

import numpy as np

olid_train = pd.read_csv('experiments/OLID/olid_train.csv', sep="\t")
olid_test = pd.read_csv('experiments/OLID/olid_test.csv', sep="\t")

olid_train = olid_train.rename(columns={'Text': 'text', 'Class': 'labels'})
olid_test = olid_test.rename(columns={'Text': 'text', 'Class': 'labels'})

olid_train['labels'] = encode(olid_train["labels"])
test_sentences = olid_test['text'].tolist()

test_preds = np.zeros((len(olid_test), args["n_fold"]))

for i in range(args["n_fold"]):
    olid_train, olid_validation = train_test_split(olid_train, test_size=0.2, random_state=args["manual_seed"])
    model = OffensiveNNModel(model_type_or_path="cnn2D", embedding_model_name="word2vec-google-news-300", train_df=olid_train, args=args, eval_df=olid_validation)
    model.train_model()
    print("Finished Training")
    model = OffensiveNNModel(model_type_or_path=args["best_model_dir"])
    predictions, raw_outputs = model.predict(test_sentences)
    test_preds[:, i] = predictions
    print("Completed Fold {}".format(i))


final_predictions = []
for row in test_preds:
    row = row.tolist()
    final_predictions.append(int(max(set(row), key=row.count)))

olid_test['predictions'] = final_predictions

olid_test['predictions'] = decode(olid_test['predictions'])
print_information(olid_test, "predictions", "labels")


from tensorflow.keras.layers import Wrapper
from tensorflow.keras import backend as K


class DropConnect(Wrapper):
    def __init__(self, layer, prob=1., **kwargs):
        self.prob = prob
        self.layer = layer
        super(DropConnect, self).__init__(layer, **kwargs)
        if 0. < self.prob < 1.:
            self.uses_learning_phase = True

    def build(self, input_shape):
        if not self.layer.built:
            self.layer.build(input_shape)
            self.layer.built = True
        super(DropConnect, self).build()

    def compute_output_shape(self, input_shape):
        return self.layer.compute_output_shape(input_shape)

    def call(self, x):
        if 0. < self.prob < 1.:
            self.layer.kernel = K.in_train_phase(K.dropout(self.layer.kernel, self.prob), self.layer.kernel)
            self.layer.bias = K.in_train_phase(K.dropout(self.layer.bias, self.prob), self.layer.bias)
        return self.layer.call(x)



import logging
import random
import shutil

import tensorflow as tf
import gensim.downloader as api
from tensorflow import keras

import numpy as np
import os
import absl.logging

from tensorflow.python.keras.callbacks import ModelCheckpoint, ReduceLROnPlateau, EarlyStopping


from offensive_nn.model_args import ModelArgs
from offensive_nn.models.offensive_cnn1d_model import OffensiveCNN1DModel
from offensive_nn.models.offensive_cnn2d_model import OffensiveCNN2DModel
from offensive_nn.models.offensive_lstm_model import OffensiveLSTMModel

logging.basicConfig()
logging.root.setLevel(logging.INFO)


absl.logging.set_verbosity(absl.logging.ERROR)

logger = logging.getLogger(__name__)


class OffensiveNNModel:
    def __init__(self, model_type_or_path,
                 embedding_model_name=None,
                 train_df=None,
                 eval_df=None,
                 args=None):

        if os.path.isdir(model_type_or_path):
            self.model = keras.models.load_model(model_type_or_path)
            self.args = self._load_model_args(model_type_or_path)

        else:
            self.train_df = train_df
            self.eval_df = eval_df

            self.args = self._load_model_args(model_type_or_path)

            if isinstance(args, dict):
                self.args.update_from_dict(args)
            elif isinstance(args, ModelArgs):
                self.args = args

            if self.args.manual_seed:
                random.seed(self.args.manual_seed)
                np.random.seed(self.args.manual_seed)
                tf.random.set_seed(self.args.manual_seed)

            self.train_texts, self.train_labels = self._prepare_data(self.train_df)
            self.eval_texts, self.eval_labels = self._prepare_data(self.eval_df)

            self.vectorizer = tf.keras.layers.TextVectorization(max_tokens=self.args.max_features, output_sequence_length=self.args.max_len)
            self.train_ds = tf.data.Dataset.from_tensor_slices(self.train_texts).batch(self.args.train_batch_size)
            self.vectorizer.adapt(self.train_ds)

            voc = self.vectorizer.get_vocabulary()
            self.word_index = dict(zip(voc, range(len(voc))))

            self.args.max_features = len(self.word_index) + 1

            self.embedding_model = api.load(embedding_model_name)
            self.embedding_matrix = self.get_emb_matrix(self.word_index, self.args.max_features, self.embedding_model)

            MODEL_CLASSES = {
                "cnn1D": OffensiveCNN1DModel,
                "cnn2D": OffensiveCNN2DModel,
                "lstm": OffensiveLSTMModel
            }

            self.model = MODEL_CLASSES[model_type_or_path](self.args, self.embedding_matrix).model

            opt = keras.optimizers.Adam(learning_rate=self.args.learning_rate)
            self.model.compile(loss='sparse_categorical_crossentropy', optimizer=opt, metrics=['accuracy'])

        logger.info(self.model.summary())

    def train_model(self,
                    args=None,
                    verbose=1):
        if os.path.exists(self.args.cache_dir) and os.path.isdir(self.args.cache_dir):
            shutil.rmtree(self.args.cache_dir)

        if os.path.exists(self.args.best_model_dir) and os.path.isdir(self.args.best_model_dir):
            shutil.rmtree(self.args.best_model_dir)

        if args:
            self.args.update_from_dict(args)
        callbacks = []

        if self.args.save_best_model:
            checkpoint = ModelCheckpoint(self.args.cache_dir, monitor='val_loss', verbose=verbose, save_best_only=True, mode='min')
            callbacks.append(checkpoint)

        if self.args.reduce_lr_on_plateau:
            reduce_lr = ReduceLROnPlateau(monitor='val_loss', factor=self.args.reduce_lr_on_plateau_factor,
                                          patience=self.args.reduce_lr_on_plateau_patience,
                                          min_lr=self.args.reduce_lr_on_plateau_min_lr, verbose=1)
            callbacks.append(reduce_lr)

        if self.args.early_stopping:
            earlystopping = EarlyStopping(monitor='val_loss', min_delta=self.args.early_stopping_min_delta,
                                          patience=self.args.early_stopping_patience, verbose=verbose, mode='auto')
            callbacks.append(earlystopping)

        x_train = self.vectorizer(np.array([[s] for s in self.train_texts])).numpy()
        x_val = self.vectorizer(np.array([[s] for s in self.eval_texts])).numpy()

        y_train = np.array(self.train_labels)
        y_val = np.array(self.eval_labels)

        self.model.fit(x_train, y_train, batch_size=self.args.train_batch_size,
                               epochs=self.args.num_train_epochs, validation_data=(x_val, y_val),
                               verbose=verbose, callbacks=callbacks)

        self.model = keras.models.load_model(self.args.cache_dir)
        self.save_model()

    def predict(self, texts):
        predictions = self.model.predict(texts, batch_size=self.args.test_batch_size)
        return np.argmax(predictions, axis=1).tolist(), predictions.tolist()

    def save_model(self):
        os.makedirs(self.args.best_model_dir, exist_ok=True)

        inputs = tf.keras.Input(shape=(1,), dtype="string")
        # Turn strings into vocab indices
        indices = self.vectorizer(inputs)
        # Turn vocab indices into predictions
        outputs = self.model(indices)

        end_to_end_model = tf.keras.Model(inputs, outputs)
        end_to_end_model.compile(
            loss='sparse_categorical_crossentropy',
            optimizer='adam',
            metrics=['accuracy']
        )
        end_to_end_model.save(self.args.best_model_dir)
        self.save_model_args(self.args.best_model_dir)

    @staticmethod
    def _prepare_data(data_frame):
        texts = []
        labels = []
        for index, row in data_frame.iterrows():
            texts.append(row['text'])
            labels.append(row['labels'])

        return texts, labels

    def save_model_args(self, output_dir):
        os.makedirs(output_dir, exist_ok=True)
        self.args.save(output_dir)

    @staticmethod
    def _load_model_args(input_dir):
        args = ModelArgs()
        args.load(input_dir)
        return args

    @staticmethod
    def load_word_emb(word_index, emebdding_model):
        embeddings_index = dict()
        for idx, key in enumerate(emebdding_model.key_to_index):
            if key in word_index:
                embeddings_index[key] = emebdding_model[key]
        return embeddings_index

    def get_emb_matrix(self, word_index, max_features, embedding_file):
        embeddings_index = self.load_word_emb(word_index, embedding_file)
        all_embs = np.stack(list(embeddings_index.values()))
        emb_mean, emb_std = all_embs.mean(), all_embs.std()
        embed_size = all_embs.shape[1]

        embedding_matrix = np.random.normal(emb_mean, emb_std, (max_features, embed_size))
        embedding_count = 0
        no_embedding_count = 0
        for word, i in word_index.items():
            if i >= max_features: continue
            embedding_vector = embeddings_index.get(word)
            if embedding_vector is not None:
                embedding_matrix[i] = embedding_vector
                embedding_count = embedding_count + 1
            else:
                no_embedding_count = no_embedding_count + 1

        no_embedding_rate = no_embedding_count/ (embedding_count + no_embedding_count)
        logger.warning("Embeddings are not found for {:.2f}% words.".format(no_embedding_rate*100))

        return embedding_matrix



args = {
    'output_dir': 'temp_olid/outputs/',
    "best_model_dir": "temp_olid/outputs/best_model",
    'cache_dir': 'temp_olid/cache_dir/',

    "early_stopping": True,
    "early_stopping_min_delta" : 0.0001,
    "early_stopping_patience": 10,

    'embed_size': 300,

    'learning_rate': 1e-3,
    'max_features': None,
    'max_len': 256,

    'manual_seed': 777,

    'n_fold': 3,

    'num_classes': 2,
    'num_train_epochs': 50,

    'reduce_lr_on_plateau': True,
    'reduce_lr_on_plateau_factor': 0.6,
    'reduce_lr_on_plateau_patience':  2,
    'reduce_lr_on_plateau_min_lr':  0.0001,

    'save_best_model': True,

    'test_batch_size': 128,
    'train_batch_size': 128,

}


import tensorflow as tf
from tensorflow.keras import layers
from tensorflow import keras


class OffensiveCNN1DModel:
    def __init__(self, args, embedding_matrix=None):

        int_sequences_input = keras.Input(shape=(None,), dtype="int64")
        embedded_sequences = layers.Embedding(args.max_features, args.embed_size,
                             embeddings_initializer=keras.initializers.Constant(embedding_matrix), trainable=False,
                             name="embedding_layer")(int_sequences_input)
        x = layers.Conv1D(128, 5, activation="relu")(embedded_sequences)
        x = layers.MaxPooling1D(5)(x)
        x = layers.Conv1D(128, 5, activation="relu")(x)
        x = layers.MaxPooling1D(5)(x)
        x = layers.Conv1D(128, 5, activation="relu")(x)
        x = layers.GlobalMaxPooling1D()(x)
        x = layers.Dense(128, activation="relu")(x)
        x = layers.Dropout(0.5)(x)
        preds = layers.Dense(args.num_classes, activation="softmax", name="dense_predictions")(x)
        self.model = keras.Model(int_sequences_input, preds, name="cnn1D_model")



import tensorflow.keras.backend as K
from tensorflow import initializers
from tensorflow.python.keras import regularizers, constraints
from tensorflow.python.keras.layers import Activation
from tensorflow.python.layers.base import Layer


class Capsule(Layer):
    def __init__(self, num_capsule, dim_capsule, routings=3, kernel_size=(9, 1), share_weights=True,
                 activation='default', **kwargs):
        super(Capsule, self).__init__(**kwargs)
        self.num_capsule = num_capsule
        self.dim_capsule = dim_capsule
        self.routings = routings
        self.kernel_size = kernel_size
        self.share_weights = share_weights
        if activation == 'default':
            self.activation = squash
        else:
            self.activation = Activation(activation)

    def build(self, input_shape):
        super(Capsule, self).build(input_shape)
        input_dim_capsule = input_shape[-1]
        if self.share_weights:
            self.W = self.add_weight(name='capsule_kernel',
                                     shape=(1, input_dim_capsule,
                                            self.num_capsule * self.dim_capsule),
                                     # shape=self.kernel_size,
                                     initializer='glorot_uniform',
                                     trainable=True)
        else:
            input_num_capsule = input_shape[-2]
            self.W = self.add_weight(name='capsule_kernel',
                                     shape=(input_num_capsule,
                                            input_dim_capsule,
                                            self.num_capsule * self.dim_capsule),
                                     initializer='glorot_uniform',
                                     trainable=True)

    def call(self, u_vecs):
        if self.share_weights:
            u_hat_vecs = K.conv1d(u_vecs, self.W)
        else:
            u_hat_vecs = K.local_conv1d(u_vecs, self.W, [1], [1])

        batch_size = K.shape(u_vecs)[0]
        input_num_capsule = K.shape(u_vecs)[1]
        u_hat_vecs = K.reshape(u_hat_vecs, (batch_size, input_num_capsule, self.num_capsule, self.dim_capsule))
        u_hat_vecs = K.permute_dimensions(u_hat_vecs, (0, 2, 1, 3))
        # final u_hat_vecs.shape = [None, num_capsule, input_num_capsule, dim_capsule]

        b = K.zeros_like(u_hat_vecs[:, :, :, 0])  # shape = [None, num_capsule, input_num_capsule]
        for i in range(self.routings):
            b = K.permute_dimensions(b, (0, 2, 1))  # shape = [None, input_num_capsule, num_capsule]
            c = K.softmax(b)
            c = K.permute_dimensions(c, (0, 2, 1))
            b = K.permute_dimensions(b, (0, 2, 1))
            outputs = self.activation(K.batch_dot(c, u_hat_vecs, [2, 2]))
            if i < self.routings - 1:
                b = K.batch_dot(outputs, u_hat_vecs, [2, 3])

        return outputs

    def compute_output_shape(self, input_shape):
        return None, self.num_capsule, self.dim_capsule


# https://www.kaggle.com/qqgeogor/keras-lstm-attention-glove840b-lb-0-043
class Attention(Layer):
    def __init__(self, step_dim,
                 W_regularizer=None, b_regularizer=None,
                 W_constraint=None, b_constraint=None,
                 bias=True, **kwargs):
        self.supports_masking = True
        self.init = initializers.get('glorot_uniform')

        self.W_regularizer = regularizers.get(W_regularizer)
        self.b_regularizer = regularizers.get(b_regularizer)

        self.W_constraint = constraints.get(W_constraint)
        self.b_constraint = constraints.get(b_constraint)

        self.bias = bias
        self.step_dim = step_dim
        self.features_dim = 0
        super(Attention, self).__init__(**kwargs)

    def build(self, input_shape):
        assert len(input_shape) == 3

        self.W = self.add_weight((input_shape[-1],),
                                 initializer=self.init,
                                 name='{}_W'.format(self.name),
                                 regularizer=self.W_regularizer,
                                 constraint=self.W_constraint)
        self.features_dim = input_shape[-1]

        if self.bias:
            self.b = self.add_weight((input_shape[1],),
                                     initializer='zero',
                                     name='{}_b'.format(self.name),
                                     regularizer=self.b_regularizer,
                                     constraint=self.b_constraint)
        else:
            self.b = None

        self.built = True

    def compute_mask(self, input, input_mask=None):
        return None

    def call(self, x, mask=None):
        features_dim = self.features_dim
        step_dim = self.step_dim

        eij = K.reshape(K.dot(K.reshape(x, (-1, features_dim)),
                              K.reshape(self.W, (features_dim, 1))), (-1, step_dim))

        if self.bias:
            eij += self.b

        eij = K.tanh(eij)

        a = K.exp(eij)

        if mask is not None:
            a *= K.cast(mask, K.floatx())

        a /= K.cast(K.sum(a, axis=1, keepdims=True) + K.epsilon(), K.floatx())

        a = K.expand_dims(a)
        weighted_input = x * a
        return K.sum(weighted_input, axis=1)

    def compute_output_shape(self, input_shape):
        return input_shape[0], self.features_dim


def squash(x, axis=-1):
    s_squared_norm = K.sum(K.square(x), axis, keepdims=True)
    scale = K.sqrt(s_squared_norm + K.epsilon())
    return x / scale



from sklearn.metrics import recall_score, precision_score, f1_score


def print_information(df, pred_column, real_column):
    predictions = df[pred_column].tolist()
    real_values = df[real_column].tolist()

    labels = set(predictions)

    for label in labels:
        print()
        print("Stat of the {} Class".format(label))
        print("Recall {}".format(recall_score(real_values, predictions, labels=labels, pos_label=label)))
        print("Precision {}".format(precision_score(real_values, predictions, labels=labels, pos_label=label)))
        print("F1 Score {}".format(f1_score(real_values, predictions, labels=labels, pos_label=label)))

    print()
    print("Weighted Recall {}".format(recall_score(real_values, predictions, average='weighted')))
    print("Weighted Precision {}".format(precision_score(real_values, predictions, average='weighted')))
    print("Weighter F1 Score {}".format(f1_score(real_values, predictions, average='weighted')))

    print("Macro F1 Score {}".format(f1_score(real_values, predictions, average='macro')))



import tensorflow as tf
from tensorflow import keras
from tensorflow.keras import layers


class OffensiveLSTMModel:

    def __init__(self, args, embedding_matrix):
        inp = tf.keras.Input(shape=(None,), dtype="int64", name="input")
        x = layers.Embedding(args.max_features, args.embed_size,
                             embeddings_initializer=keras.initializers.Constant(embedding_matrix), trainable=False,
                             name="embedding_layer")(inp)
        x = layers.Bidirectional(layers.LSTM(64, return_sequences=True, name="lstm_1"))(x)
        x = layers.Bidirectional(layers.LSTM(64, name="lstm_2"))(x)
        x = layers.Dense(256, activation="relu", name="dense_1")(x)
        x = layers.Dense(args.num_classes, activation="softmax", name="dense_predictions")(x)
        self.model = tf.keras.Model(inputs=inp, outputs=x, name="lstm_model")



from tensorflow.python.keras import Input, Model
from tensorflow.python.keras.layers import Embedding, SpatialDropout1D, Bidirectional, GRU, Flatten, Dense, Dropout, \
    BatchNormalization
from tensorflow.python.ops.init_ops_v2 import glorot_normal
from tensorflow.python.ops.initializers_ns import orthogonal

from offensive_nn.models.layers import Capsule


class OffensiveCapsuleModel:
    def __init__(self, args, embedding_matrix):
        inp = Input(shape=(args.max_len,))

        x = Embedding(args.max_features, args.embed_size, weights=[embedding_matrix], trainable=False)(inp)

        x = SpatialDropout1D(rate=0.2)(x)
        x = Bidirectional(GRU(100, return_sequences=True,
                              kernel_initializer=glorot_normal(seed=12300),
                              recurrent_initializer=orthogonal(gain=1.0, seed=10000)))(x)

        x = Capsule(num_capsule=10, dim_capsule=10, routings=4, share_weights=True)(x)
        x = Flatten()(x)

        x = Dense(100, activation="relu", kernel_initializer=glorot_normal(seed=12300))(x)
        x = Dropout(0.12)(x)
        x = BatchNormalization()(x)

        x = Dense(args.num_classes, activation="sigmoid")(x)
        self.model = Model(inputs=inp, outputs=x)
        # model.compile(loss='binary_crossentropy', optimizer=Adam(), metrics=['accuracy'])



import json
import os
from dataclasses import asdict, field, dataclass


@dataclass
class ModelArgs:

    best_model_dir: str = "outputs/best_model"
    cache_dir: str = "cache_dir/"
    embed_size: int = 300

    early_stopping: bool = True
    early_stopping_min_delta: float = 0.0001
    early_stopping_patience: int = 10

    learning_rate: float = 1e-3

    manual_seed: int = None

    max_features: int = None
    max_len: int = 256

    not_saved_args: list = field(default_factory=list)
    num_classes: int = 2
    num_train_epochs: int = 50

    reduce_lr_on_plateau: bool = True
    reduce_lr_on_plateau_factor: float = 0.6
    reduce_lr_on_plateau_patience: int = 2
    reduce_lr_on_plateau_min_lr: float = 0.0001



    save_best_model: bool = True

    test_batch_size: int = 128
    train_batch_size: int = 128

    def update_from_dict(self, new_values):
        if isinstance(new_values, dict):
            for key, value in new_values.items():
                setattr(self, key, value)
        else:
            raise (TypeError(f"{new_values} is not a Python dict."))

    def get_args_for_saving(self):
        args_for_saving = {key: value for key, value in asdict(self).items() if key not in self.not_saved_args}
        return args_for_saving

    def save(self, output_dir):
        os.makedirs(output_dir, exist_ok=True)
        with open(os.path.join(output_dir, "model_args.json"), "w") as f:
            json.dump(self.get_args_for_saving(), f)

    def load(self, input_dir):
        if input_dir:
            model_args_file = os.path.join(input_dir, "model_args.json")
            if os.path.isfile(model_args_file):
                with open(model_args_file, "r") as f:
                    model_args = json.load(f)

                self.update_from_dict(model_args)


from setuptools import find_packages, setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="offensivenn",
    version="0.0.4",
    author="Tharindu Ranasinghe",
    author_email="rhtdranasinghe@gmail.com",
    description="Offensive Language Identification with Neural Networks",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TharinduDR/https://github.com/TharinduDR/OffensiveNN",
    packages=find_packages(exclude=("experiments", )),
    classifiers=[
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3",
        "Topic :: Scientific/Engineering :: Artificial Intelligence",
    ],
    python_requires=">=3.6",
    install_requires=[
        "pandas",
        "numpy",
        "gensim",
        "sklearn"
    ],
)



from sklearn.metrics import f1_score, recall_score, precision_score


def macro_f1(y_true, y_pred):
    return f1_score(y_true, y_pred, average='macro')


def weighted_f1(y_true, y_pred):
    return f1_score(y_true, y_pred, average='weighted')


# def recall(y_true, y_pred):
#     y_true = decode(y_true)
#     y_pred = decode(y_pred)
#     return recall_score(y_true, y_pred, labels=labels, pos_label=pos_label)
#
#
# def precision(y_true, y_pred):
#     y_true = decode(y_true)
#     y_pred = decode(y_pred)
#     return precision_score(y_true, y_pred, labels=labels, pos_label=pos_label)
